﻿using Microsoft.EntityFrameworkCore;
using System.Reflection.Metadata;

namespace EFCoreExample.Models
{
    public class CompanyContext : DbContext
    {
        public CompanyContext(DbContextOptions<CompanyContext> options) : base(options)
        {
        }

        public DbSet<Department> Department { get; set; }
        public DbSet<Employee> Employee { get; set; }
        public DbSet<Information> Information { get; set; }
        
        public DbSet<City> City { get; set; }

        public DbSet<EmpDep> EmpDep { get; set; }

        public DbSet<DepartmentWithMultipleEmployee> DepartmentWithMultipleEmployee { get; set; }

        //public DbSet<Dictionary<string, object>> CityDictionary => Set<Dictionary<string, object>>("City");

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Department>(entity =>
            {
                entity.Property(e => e.Name)
                    .IsRequired()
                    .HasMaxLength(50)
                    .IsUnicode(false);
            });

            modelBuilder.Entity<Employee>(entity =>
            {
                entity.Property(e => e.Designation)
                    .IsRequired()
                    .HasMaxLength(25)
                    .IsUnicode(false);

                entity.Property(e => e.Name)
                    .IsRequired()
                    .HasMaxLength(100)
                    .IsUnicode(false);

                entity.HasOne(d => d.Department)
                    .WithMany(p => p.Employee)
                    .HasForeignKey(d => d.DepartmentId)
                    .OnDelete(DeleteBehavior.Cascade)
                    .HasConstraintName("FK_Employee_Department");
            });

            modelBuilder.Entity<EmpDep>().ToView("EmpDepView");

            modelBuilder.Entity<DepartmentWithMultipleEmployee>().HasNoKey().ToFunction("DepartmentWithMultipleEmployee");

            /*modelBuilder.SharedTypeEntity<Dictionary<string, object>>(
            "City", bb =>
            {
                bb.HasNoKey();
                bb.Property<int>("Id");
                bb.Property<string>("Name");
                bb.Property<string>("Population");
            });*/
        }
    }
}